/*
 * gnu.ldml.Element Copyright (C) 2004 Free Software Foundation,
 * Inc.
 *
 * This file is part of GNU Classpath.
 *
 * GNU Classpath is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2, or (at your option) any later version.
 *
 * GNU Classpath is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * GNU Classpath; see the file COPYING. If not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA.
 */
package gnu.ldml;

import java.util.ArrayList;

public class Element
{
  public static final Element ROOT = new Element();
  public ArrayList children = new ArrayList();
  public String defaultType;
  public String displayName;
  public Parser parentParser;
  public String qualifiedName;
  public Element superElement;

  private Element()
  {
  }

  public Element(Parser p, Element parent, String name)
  {
    parentParser = p;
    superElement = parent;
    qualifiedName = name;
    if (parent != ROOT)
      parent.addChild(this);
  }

  public void addChild(Element e)
  {
    children.add(e);
  }

  public String getFullName()
  {
    if (superElement == ROOT)
      return qualifiedName;
    return superElement.getFullName() + "." + qualifiedName;
  }
}
